/*

  (C) Marco Falcioni UCLA 1998

  this routine takes a raw RP with a list of 
  computed intensies and tries to find the spots which are 
  related by simmetry in an empirical fashion.  It is unelegant but 
  I'd rather do this and not input 230 space groups.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <malloc.h>

#include "error.h"
#include "constants.h"
#include "memory.h"

#include "cell.h"
#include "misc.h"
#include "rad_pattern.h"
#include "spot.h"
#include "intensity.h"


void find_composites(struct Rad_pattern * RP, double separation_DEG) 
{
  struct Spot * first;
  long i;

  if(RP == 0) {
    push_generic_error("find_composites", "(nil) arg passed, you dope");
    exit_clean();
  }

/*****************************************************************************\
 *                                                                           *
 *    Look for angles closer than the given separation                       *
 *    Note that we never compose the last spot                               *
 *    there are two states: looking for first or looking for next            *
 *                                                                           *
\*****************************************************************************/

  first = (struct Spot *) 0;

  for(i = (long) RP->length - 1; i > 0 ; --i) {
    struct Spot * this = RP->I[i];
    struct Spot * next = RP->I[i-1];
    int close;

#ifdef NOCANC    
    switch (RP->type) {
    case XRAY:
      if(this->xray->value < tiny) {first = 0; continue;}
      break;
    case NEUTRON:
      if(this->neutron->value < tiny) {first = 0; continue;}
      break;
    case BOTH:
      if(this->xray->value < tiny && this->neutron->value < tiny) {first = 0; continue;}
      break;
    case NONE:
      default:
	/* error condition */
	push_generic_error("find_composites", "weird enum!");
	exit_clean();
    }
#endif
    
    close = (fabs(this->twotheta - next->twotheta) < separation_DEG);

    if(close) {
      if(!first) first = this;

      if(RP->type == XRAY || RP->type == BOTH) {
	first->xray->value += next->xray->value;
	next->xray->value = -1;
	next->xray->composite = this->xray;
      }
      
      if(RP->type == NEUTRON || RP->type == BOTH) {
	first->neutron->value += next->xray->value;
	next->neutron->value = -1;
	next->neutron->composite = this->neutron;
      }
      
    } else {
      first = (struct Spot *) 0;
    }     
  }


  return;

}
