/*

  Marco Falcioni (C) 1997

  grid for quick access of neighboring atoms
  a grid is an ensemble of boxes
  a box is a chain, which contains atoms. a box also contains 
  the 8 neighboring boxses.


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __grid_h
#define __grid_h

/* includes */

#include <stdio.h>

struct Box;
struct Cell;
struct Positions;
struct Atom;
struct Params;
/* data */

struct Grid {
  struct Box ** box_list;
  double corner[3];
  double center[3];
  double spacing;
  double inv_spacing;
  unsigned int sizes[3]; /* how many boxes on each coner */
  unsigned int cells[3]; /* how many cells fit in 1 radius */
  unsigned int n_boxes;
  unsigned filled;
};

/* function prototypes */


/* 
   all I need to make a grid is how big I need it to be 
   that info is contained in the Cell structure
*/

struct Grid * new_Grid (const struct Params * P, const struct Cell * C, 
			unsigned int radius);

void delete_Grid (struct Grid * G);

void print_Grid (const struct Grid * G, FILE * stream);


/* this returns the box and the indeces of the box, unless
   we pass i = 0, in which case 0 is returned */
struct Box * index_atom_Grid (const struct Grid * G, 
			      struct Atom * A,
			      const struct Cell * C, long * i);

void untag_Grid(struct Grid * G);

void clean_Grid(struct Grid * G);

void fill_Grid(struct Grid * G, 
	     struct Positions * T, 
	     const struct Cell * C);

#endif
