/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/
#include <stdio.h>
#include <stdlib.h>

#include "error.h"
#include "resources.h"
#include "stringlib.h"

#include "misc.h"
#include "functions.h"
#include "grid.h"
#include "chain.h"
#include "link.h"
#include "atoms.h"
#include "spline.h"
#include "cell.h"
#include "symmetry.h"
#include "grid.h"
#include "params.h"
#include "rad_pattern.h"
#include "oogl.h"
#include "positions.h"
#include "verbose.h"
#include "conf.h"

int main (int argc, char ** argv)
{
  struct Params * P = 0;
  struct Cell * C = 0;
  struct Positions * T = 0;
  struct Sym_group * G = 0;
  struct Grid * Gr;
  struct Conf * conf;
  unsigned long i=0;
  struct string_list * options;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  P = read_Params(stdin);

  T = init_Positions(stdin);

  clone_Positions(T,T);
  
  C = read_Cell(stdin);

  G = read_Sym_group(T->n_symmetry, stdin);
  
  test_Sym_group(G);

  P->Bond = read_Spline(P->Bond_Spl_file);
  P->NoBond = read_Spline(P->NoBond_Spl_file);
  P->Angle = read_Spline(P->Angle_Spl_file);
  P->AvAngle = read_Spline(P->AvAngle_Spl_file);

  conf = new_Conf(T->n_unique);
  Gr = new_Grid(P, C, 0);

  while(!feof(stdin)) {
    i++;

    if(read_Conf(conf, stdin) == -1) break;

    pcopy_Conf(conf, T);
    printf("# conf %ld\n",i);
    oogl_Init_list(stdout);                    

    symmetrize_Positions(T, G);
    mod_Positions(T);
    merge_positions(T, P, C);
    clean_Grid(Gr);
    fill_Grid(Gr, T, C);
    oogl_Cell(C,0); 
    oogl_bonds(P, C, T, Gr,0);
    oogl_Positions(T, C, 1);
    oogl_Finish_list();

  }


  return 0;
  
}

