/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/
#include <stdio.h>
#include <stdlib.h>

#include "error.h"
#include "resources.h"
#include "stringlib.h"

#include "functions.h"
#include "positions.h"
#include "grid.h"
#include "chain.h"
#include "link.h"
#include "atoms.h"
#include "params.h"
#include "oogl.h"
#include "misc.h"
#include "verbose.h"

#define OOGL

int main (int argc, char ** argv)
{
  struct Params * P;
  struct Cell * C = 0;
  struct Rad_pattern * RP;
  struct Positions * T = 0;
  struct Sym_group * G = 0;
  struct Grid * Gr = 0;
  FILE * a;
  struct string_list * options;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  read_data(&P, &T, &C, &G, &RP, stdin);

  srand48(P->seed);

  Gr = new_Grid(P,C,0);

  symmetrize_Positions(T,G);
  mod_Positions(T);
  merge_positions(T,P,C);
  mod_Positions(T);

  fill_Grid(Gr, T,C);

#ifdef OOGL
  a = safe_open("cell.oogl","w");
  oogl_Init_list(a);
  oogl_Cell(C,0);
  oogl_all_bonds(P,C,T,Gr,1);
  oogl_Grid(Gr, C);
  oogl_Finish_list();
  fclose(a);
#endif
  
  return 0;
  
}

