/*

  (C) 1997 Marco Falcioni UCLA

 
  this is a container for simulation parameters which 
  i need here and there in the simulation.


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/


#include <stdio.h>

#ifndef __params_h
#define __params_h 

struct Spline;
struct Radius;
struct SP_entry;

struct Params {

  unsigned long n_actual; 
  /* parameter: the actual number of T atoms (framework density) */
  
  /* parameter: number of fixed temperature MC sweeps */
  unsigned long sw_init;  /* used to find melting temperature */
  unsigned long sw_therm; /* thermalization: lose track of initial condition */
  unsigned long sw_anneal; /* annealing steps */
  unsigned long granularity; /* unit of MC steps for controlling 
				acceptance ratio */
  double bond;     /* the maximum distance below which atoms bond */
  double bond2;    
  double frange;   /* the maximum distance below which unbonded atoms repel */
  double frange2;  
  double merge2;  /* the maximum distance below which atoms can merge */
  double ene_merge;  /* the minimum energy for merging */

  double r_max;  /* initial amplitude parameter */
  double r_min;  /* initial amplitude parameter */
  double gamma;  /* proportional contol parameter */
  double t_max;  /* runtime: simulated annealing: hot temperature */
  double t_min;  /* parameter: simulated annealing: final cold temperature */
  double t_0;  /* Parameter: simulated annealing: step temperature */
  double alpha;  /* factor by which temperature is reduced */
  double melting_frac;  /* this defines the molten phase */
  double mc_frac;  /* this defines move amplitude */

  double radius;  /* runtime: the amount by which atoms are moved */
  double temperature;  /* runtime: temperature */

  /* these are the weights we use for the energy Parameters*/
  double wt_radpat;
  double wt_bond;
  double wt_nobond;
  double wt_angl;
  double wt_aang;
  double wt_merge;
  double wt_unbond;

  struct SP_entry ** SP; /* the scattering data */

  /* these are the four splines for the energy spectra */  
  struct Spline * Bond;
  struct Spline * NoBond;
  struct Spline * Angle;
  struct Spline * AvAngle;

  /* and the filenames */
  char * Bond_Spl_file;
  char * NoBond_Spl_file;
  char * Angle_Spl_file;
  char * AvAngle_Spl_file;
  char * Scat_Fact_file;
  char * Rad_file;

  /* the random seed, or whatever we need for the random routine */

  long seed;

  /* 
     the path to the data files; this is read from an environment variable
     in DATAPATH as defined above
   */

  char * data_path;
  
};


struct Params * read_Params(FILE * stream);
void write_Params(const struct Params * P, FILE * stream);

int set_radius(struct Params * P, double fraction);
/* changes the temperature T' = alpha T and returns Delta T */
double scale_temperature(struct Params * P);
void delete_Params(struct Params * P);

#endif
