/*

  (C) 1998 Marco Falcioni UCLA

   (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <errno.h>

#include "registry.h"
#include "conf.h"

extern int errno;

#include "error.h"

#include "misc.h"


struct Registry * new_Registry(void)
{
  struct Registry * me;

  me = (struct Registry *) Malloc (sizeof(struct Registry));

  /* do some initialization */

  me->head = 0;
  me->length = 0;

  return me;
}

void delete_Registry(struct Registry * me)
{
  if(me == 0) return;
  
  while (me->length > 0) {
    struct Conf * delenda;
    me->length --;
    delenda = me->head;
    me->head = delenda->next;
    delete_Conf(delenda);
  }
  
  Free(me);

  return;
}

void print_Registry(struct Registry * me, FILE * stream)
{
  struct Conf * scribenda;
#ifndef NOCHECKS
  if(me == 0 || stream == 0) {
    push_generic_error("print_Registry","(nil) args passed");
    exit_clean();
  }
#endif
  for(scribenda = me->head; scribenda != 0; scribenda = scribenda->next){
    print_Conf(scribenda, stream);
  }
  
  if(ferror(stream)){
    char buf[1024];
    sprintf(buf, "error while writing to stream: %s", strerror(errno));
    push_generic_error("print_Registry",buf);
    exit_clean();
  }

}   


void push_Registry(struct Conf * addenda, struct Registry * on_me)
{ 
  struct Conf * runner;
#ifndef NOCHECKS 
  if(addenda == 0 || on_me == 0) {
    push_generic_error("push_Registry","(nil) args passed");
    exit_clean();
  }
#endif

#ifdef DEBUG
  printf("length %d\n", on_me->length);
#endif

  if(on_me->head == 0) {
    on_me->head = addenda;
    addenda->next = 0;
  } else {
    struct Conf * prev = 0;
    runner = on_me->head;
#ifdef DEBUG
    printf("1 add %p ene %f run %p ene %f\n", addenda, addenda->tot_energy,
	   runner, runner->tot_energy);
    fflush(stdout);
#endif
    while (runner != 0 && addenda->tot_energy > runner->tot_energy) {
#ifdef DEBUG
    printf("2 add %p ene %f run %p ene %f\n", addenda, addenda->tot_energy,
	   runner, runner->tot_energy);
    fflush(stdout);
#endif
      prev = runner;
      runner = runner->next;
    }
    if(prev == 0) {
      on_me->head = addenda;
    } else {
      prev->next = addenda; 
    }
    addenda->next = runner;

  }

  on_me->length ++;

  return;
}


struct Conf * pop_Registry(struct Registry * me)
{
  struct Conf * redenda;

#ifndef NOCHECKS
  if(me == 0) {
    push_generic_error("pop_Registry","(nil) args passed");
    exit_clean();
  }
#endif

  redenda = me->head;
  if(me->length == 1) {
    me->head = 0;
    me->length = 0;
  } else if (me->length > 1) {
    me->head = redenda->next;
    me->length--;
  }
  return redenda;

}


#ifdef TEST
/* test program */

int main (void) {
  struct Conf * me;
  struct Registry * R;
  unsigned i;
  FILE * ene;

  ene = safe_open("energies", "r");


  R = new_Registry();

  Srand(412);

  for(i = 0; i < 20; i ++) {
    me = new_Conf(2);
    read_Conf(me, ene);

    push_Registry(me, R); 
  }

  print_Registry(R, stdout);

  return 0;
}


#endif


