/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "error.h"
#include "resources.h"
#include "stringlib.h"
#include "constants.h"

#include "misc.h"
#include "functions.h"
#include "grid.h"
#include "chain.h"
#include "link.h"
#include "atoms.h"
#include "spline.h"
#include "cell.h"
#include "symmetry.h"
#include "grid.h"
#include "params.h"
#include "rad_pattern.h"
#include "oogl.h"
#include "positions.h"
#include "verbose.h"
#include "conf.h"
#include "filters.h"

int main (int argc, char ** argv)
{
  struct Params * P = 0;
  struct Cell * C = 0;
  struct Positions * T = 0;
  struct Sym_group * G = 0;
  struct Conf * conf;
  int i=0, unique = 0;
  struct string_list * options;
  double min = max_double, min_RP = max_double, min_Pos = max_double;
  FILE * Tot, * RP, * Pos;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  Tot = safe_open("tot.hist", "w");
  RP = safe_open("rp.hist", "w");
  Pos = safe_open("pos.hist", "w");

  P = read_Params(stdin);

  T = init_Positions(stdin);

  clone_Positions(T,T);

  for(i = 0; i < T->n_unique; i++) {
    if(T->sym_related[i]->head->atom->n_bonds == 4) unique++;
  }

  i = 0;
  
  C = read_Cell(stdin);

  G = read_Sym_group(T->n_symmetry, stdin);
  
  test_Sym_group(G);

  P->Bond = read_Spline(P->Bond_Spl_file);
  P->NoBond = read_Spline(P->NoBond_Spl_file);
  P->Angle = read_Spline(P->Angle_Spl_file);
  P->AvAngle = read_Spline(P->AvAngle_Spl_file);

  conf = new_Conf(T->n_unique);

  while(!feof(stdin)) {
    i++;
    if(i%100 == 0) fprintf(stderr, "100 conf read %d\n",i);
    if(read_Conf(conf, stdin) == -1) break;

    if(not_four_coordinated(conf, unique)) continue;

    if(not_merged_right(conf, (int)P->n_actual, unique)) continue;

    /*    if(conf->tot_energy < min) { 
	  min = conf->tot_energy; */
      fprintf(Tot, "%d ",i);
      fprint_Conf(conf, Tot);
      /*    } */

   /*  if(conf->RP_energy < min_RP) {
      min_RP = conf->RP_energy;*/
      fprintf(RP, "%d ",i);
      fprint_Conf(conf, RP);
  /*   }*/
    
   /*  if(conf->Pos_energy < min_Pos) {
      min_Pos = conf->Pos_energy;*/
      fprintf(Pos, "%d ",i);
      fprint_Conf(conf, Pos);
  /*   }*/
    
  }

  fclose(Pos);
  fclose(RP);
  fclose(Tot);
  return 0;
  
}
