/*

  (C) 1997 Marco Falcioni UCLA

  methods of spot object 

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "constants.h"

#include "misc.h"
#include "cell.h"
#include "spot.h"
#include "intensity.h"


struct Spot * new_Spot(enum Rad_type a)
{
  struct Spot * me;

  me = (struct Spot *) Malloc(sizeof(struct Spot));

  if(me == 0) {
    push_error("new_Spot",Error_Alloc_);
    exit_clean();
  }

  me->h_ = 0.0;
  me->k_ = 0.0;
  me->l_ = 0.0;

  me->multiplicity = 0;

  me->twotheta = 0.0;
 
  switch (a) {
  case XRAY:
    me->xray = new_Intensity();
    me->neutron = 0;
    break;
  case NEUTRON:
    me->xray = 0;
    me->neutron = new_Intensity();
    break;
  case BOTH:
    me->xray = new_Intensity();
    me->neutron = new_Intensity();
  case NONE:
    break;
  }

  return me;

}

void delete_Spot(struct Spot * me)
{
  if(me == 0) return;
  delete_Intensity(me->xray);
  delete_Intensity(me->neutron);
  Free(me);
  return;
}

void print_Spot(const struct Spot * me, FILE * stream)
{
  if(me == 0 || stream == 0) {
    push_generic_error("read_Spot","(nil) args passed");
    exit_clean();
  }
  fprintf(stream, "%2d %2d %2d %2d", 
	  (int) me->h_, (int) me->k_, (int) me->l_,
	  me->multiplicity);
  print_Intensity(me->xray, stream);
  print_Intensity(me->neutron, stream);
  fprintf(stream, "\n");
  return;
}

void copy_Spot(const struct Spot * me, struct Spot * to_me)
{
  if(me == 0 || to_me == 0) {
    push_generic_error("copy_Spot","(nil) args passed");
    exit_clean();
  }

  to_me->h_ = me->h_;
  to_me->k_ = me->k_;
  to_me->l_ = me->l_;
  
  to_me->twotheta = me->twotheta;

  to_me->multiplicity = me->multiplicity;

  if(me->xray != 0) {
    copy_Intensity(me->xray, to_me->xray);
  }

  if(me->neutron != 0) {
    copy_Intensity(me->neutron,to_me->neutron);
  }

  return;

}

/* makes a new spot if me == 0 */
struct Spot * read_Spot(struct Spot * me, const char * string, 
			enum Rad_type a) 
{
  int rs,ns;
  double ro, no;
  int c;

  if(string == 0 || me == 0) {
    push_generic_error("read_Spot","(nil) arg passed");
    exit_clean();
  }

  if((c = sscanf(string,"%lf %lf %lf %d %lf %d %lf %d",
	    &me->h_, &me->k_, &me->l_, &me->multiplicity, 
	    &ro, &rs, &no, &ns)) < 4) {
    char buf[1024];
    sprintf(buf, "error reading from string\n%s", string);
    push_generic_error("read_Spot",buf);
    exit_clean();
  }

  if(me->h_ == 0 && me->k_ == 0 && me->l_ == 0) {
    push_generic_error("read_Spot","invalid spot (000) seen");
    exit_clean();
  }

  switch (a) {
  case XRAY:
    if(c != 6) {
      push_generic_error("read_Spot","error reading xray data from string");
      exit_clean();
    }
    me->xray->value = ro;
    me->xray->sigma = rs;
    break;
  case NEUTRON:
    if(c != 6) {
      push_generic_error("read_Spot","error reading neutron data from string");
      exit_clean();
    }
    me->neutron->value = ro;
    me->neutron->sigma = rs;
    break;
  case BOTH:
    if(c != 8) {
      push_generic_error("read_Spot",
			 "error reading radiation data from string");
      exit_clean();
    }
    me->xray->value = ro;
    me->xray->sigma = rs;
    me->neutron->value = no;
    me->neutron->sigma = ns;
    break;
  case NONE:
  default:
    push_generic_error("read_Spot","unexpected switch label reached");
    exit_clean();
  }

  return me;
}
