/*

  Marco Falcioni (C) 1998 UCLA

  write a datafile in a standard XTL form, P1 symmetry

  (C) 1998 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>

#include "error.h"

#include "positions.h"
#include "cell.h"
#include "symmetry.h"
#include "misc.h"
#include "functions.h"
#include "params.h"
#include "rad_pattern.h"
#include "SP_entry.h"
#include "spline.h" 
#include "chain.h"
#include "link.h"
#include "atoms.h"

void write_P1(struct Positions * T, struct Cell * C, struct Sym_group * G, 
	       FILE * stream)
{
  int i, j = 0;

  if(stream == 0 || T == 0 || C == 0 || G == 0) {
    push_generic_error("write_XTL","(nil) args passed");
    exit_clean();
  }

  fprintf(stream, "TITLE Zeolite from Zefsa\n");

  fprintf(stream, "DIMENSION 3\n");

  write_Cell_XTL(C, stream);

  fprintf(stream, "SYMMETRY  NUMBER 1  LABEL P1\n");

  fprintf(stream, 
	  "SYM MAT  1.0  0.0  0.0  0.0  1.0  0.0  0.0  0.0  1.0 0.0000 0.0000 0.0000\n\n");

  symmetrize_Positions(T, G);
  mod_Positions(T);

  fprintf(stream,
	  "ATOMS\nNAME       X          Y          Z     CHARGE   TEMP    OCCUP   SCAT\n");

  for(i = 0; i < T->n_unique; ++i) {
    struct Link * aa;
    for(aa = T->sym_related[i]->head; aa != 0; aa = aa->next){
      fprintf(stream, "%2s%d   %8.5f    %8.5f    %8.5f  0.0000  0.0000  1.0000   %2s0+\n",
	      aa->atom->label,j, aa->atom->ra, aa->atom->rb, aa->atom->rc, 
	      aa->atom->label);
      ++j;
    }
  }

  fprintf(stream, "EOF\n");

  return;

}
